"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
// import JSON5 from 'json5';
const path_1 = require("path");
// import test from 'tape';
const fileContent = fs_1.readFileSync(path_1.resolve(__dirname, '../../data/data.json'), { encoding: 'utf8' }).toString();
const json = JSON.parse(fileContent);
// Final result code map
const codemap = {};
/**
 * Start of the Unicode Private Area
 */
const privateFrom = 0xe000;
/**
 * End of Unicode Private Area
 */
const privateTo = 0xf8ff;
/**
 * Maximum number of empty count substitute string
 */
const maxCount = privateTo - privateFrom + 1;
const privateAreaRegExp = `[\\u${privateFrom.toString(16)}-\\u${privateTo.toString(16)}]`;
if (RegExp(privateAreaRegExp).test(fileContent)) {
    throw new Error('Illegal characters fond!');
}
const isChinese = (low) => (low >= 0x4e && low <= 0x9f) || (low >= 0xf9 && low <= 0xfa);
const isEmpty = (char) => char === null || char === undefined || char === '';
const getSeparator = (map) => {
    let char;
    for (let i = 0; i < 65536; i++) {
        char = String.fromCharCode(i);
        outerLop: for (const key of Object.keys(map)) {
            for (const str of map[key]) {
                if (typeof str === 'string' && str.indexOf(char) !== -1) {
                    break outerLop;
                }
            }
            return char;
        }
    }
    throw new Error('Error getting separator');
};
const separator = getSeparator(json);
/**
 * Makes a empty count string. This is used to indicate how many items are empty between two adjacent items in the source string
 * Use characters within Unicode Private Area as a variable, for example, to mark 1 empty item, then the character is `String.fromCharCode(privateFrom + 1)`
 * If it is more than 1 symbol can handle, i.e. `number > maxCount` then use more than one symbol
 * @param num
 */
const makeEmptyCountString = (num) => {
    const extra = num - maxCount;
    const char = String.fromCharCode(num + privateFrom);
    return extra > 0 ? char + makeEmptyCountString(extra) : char;
};
/**
 * Return the number of empty items count of the encoded string
 * @param str
 */
const parseCountStr = (str) => {
    let count = 0;
    for (let i = 0; i < str.length; i++) {
        count = count * maxCount + str[i].charCodeAt(0) - privateFrom;
    }
    return count;
};
/**
 * Encoder
 */
const maxIndex = 256;
let result = '';
let emptyCount = 0;
for (let i = 0; i < maxIndex; i++) {
    if (!Array.isArray(json[i]) || !json[0].length) {
        emptyCount += maxIndex;
        continue;
    }
    for (let j = 0; j < maxIndex; j++) {
        let char = json[i][j];
        // If current character doesn't have a map, emptyCount++
        if (isEmpty(char)) {
            emptyCount++;
            continue;
        }
        else if (isChinese(i)) {
            char = char.trimRight();
        }
        // if there is at lease one empty characters, then prepend empty count string
        if (emptyCount) {
            result += makeEmptyCountString(emptyCount);
            emptyCount = 0;
        }
        result += char + separator;
    }
}
console.log(result.length, separator.length);
fs_1.writeFileSync(path_1.resolve(__dirname, 'test.txt'), result, { encoding: 'utf-8' });
process.exit();
/**
 * Decoder
 */
let indexContent = 0;
let indexCodemap = 0;
while (true) {
    let indexEndContent = result.indexOf(separator, indexContent);
    if (indexEndContent === -1) {
        indexEndContent = result.length;
    }
    const str = result.substring(indexContent, indexEndContent);
    const [emptyCountStr] = str.match(RegExp(`^${privateAreaRegExp}+`)) || [''];
    const empty = parseCountStr(emptyCountStr);
    indexCodemap += empty;
    indexContent += emptyCountStr.length;
    codemap[String.fromCharCode(indexCodemap)] = result.substring(indexContent, indexEndContent);
    indexCodemap++;
    if (indexEndContent === result.length) {
        break;
    }
    indexContent = indexEndContent + separator.length;
}
exports.charmap = codemap;
let output = '';
for (const key of Object.keys(exports.charmap)) {
    output += key + '\t' + exports.charmap[key] + '\n';
}
fs_1.writeFileSync(path_1.resolve(__dirname, '../../data/test1.json'), output);
// writeFileSync(resolve(__dirname, '../../data/test.json'), JSON.stringify(codemap).split('","').join('",\n"'));
// // be sure that json[0] exists
// for (let i = 0; i <= lastIndex; i++) {
//   if (!Array.isArray(json[i]) || json[i].length === 0) {
//     codemap.push([]);
//   } else {
//     codemap[i] = [];
//     for (let j = 0; j < json[i].length; j++) {
//       if (json[i][j] === undefined || json[i][j] === null) {
//         json[i][j] = null;
//       }
//       if (isChinese(i) && typeof json[i][j] === 'string') {
//         json[i][j] = json[i][j]!.trimRight();
//       }
//       codemap[i].push(json[i][j]);
//     }
//     while (codemap[i].length && codemap[i].lastIndexOf(null) === codemap[i].length - 1) {
//       codemap[i].pop();
//     }
//   }
// }
// const generateCode = `type Input = Array<Array<string | undefined>;
// let arr: Input = ${JSON5.stringify(codemap).replace(/null/g, '')};
// export interface Charmap {
//   [key: string]: string
// };
// export const charmap: Charmap = {};
// for (let high = 0; high < arr.length; high++) {
//   // The detection is used to fix the redundant trailing space
//   for (let low = 0; low < arr[high].length; low++) {
//     const value = arr[high][low];
//     if (typeof value === 'string' && value.length) {
//       const char = String.fromCharCode((high << 8) + low);
//       charmap[char] = value;
//     }
//   }
// }
// // Fix memory leak
// arr = undefined;
// `;
// writeFileSync(resolve(__filename, '../../data/charmap.ts'), generateCode, { encoding: 'utf8' });
